/*
 * ethermirror.{cc,hh} -- rewrites Ethernet packet a->b to b->a
 * Eddie Kohler
 *
 * Copyright (c) 2000 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, subject to the conditions
 * listed in the Click LICENSE file. These conditions include: you must
 * preserve this copyright notice, and you cannot mention the copyright
 * holders in advertising related to the Software without their permission.
 * The Software is provided WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED. This
 * notice is a summary of the Click LICENSE file; the license in that file is
 * legally binding.
 */

#include <click/config.h>
#include "ethermirror.hh"
#include <click/click_ether.h>

EtherMirror::EtherMirror()
{
  MOD_INC_USE_COUNT;
  add_input();
  add_output();
}

EtherMirror::~EtherMirror()
{
  MOD_DEC_USE_COUNT;
}

Packet *
EtherMirror::simple_action(Packet *p)
{
  click_ether *ethh = (click_ether *)p->data();
  unsigned char tmpa[6];
  tmpa = ethh->ether_dhost;
  ethh->ether_dhost = ethh->ether_shost;
  ethh->ether_shost = tmpa;
  return p;
}

EXPORT_ELEMENT(EtherMirror)
